@@ -4,12 +4,13 @@ package Data::Record::Serialize::Encode::dbi;
 
 use Moo::Role;
 
-our $VERSION = '0.08'; # TRIAL
+our $VERSION = '0.09'; # TRIAL
 
 use Data::Record::Serialize::Types -types;
 
 use SQL::Translator;
 use SQL::Translator::Schema;
+use Types::Standard -types;
 
 use List::Util qw[ pairmap ];
 
@@ -18,6 +19,24 @@ use Carp;
 
 use namespace::clean;
 
+#pod =attr C<dsn>
+#pod
+#pod I<Required> The DBI Data Source Name (DSN) passed to B<L<DBI>>.  It
+#pod may either be a string or an arrayref containing strings or arrayrefs,
+#pod which should contain key-value pairs.  Elements in the sub-arrays are
+#pod joined with C<=>, elements in the top array are joined with C<:>.  For
+#pod example,
+#pod
+#pod   [ 'SQLite', { dbname => $db } ]
+#pod
+#pod is transformed to
+#pod
+#pod   SQLite:dbname=$db
+#pod
+#pod The standard prefix of C<dbi:> will be added if not present.
+#pod
+#pod =cut
+
 has dsn => (
     is       => 'ro',
     required => 1,
@@ -42,21 +61,71 @@ has dsn => (
     },
 );
 
+#pod =attr C<table>
+#pod
+#pod I<Required> The name of the table in the database which will contain the records.
+#pod It will be created if it does not exist.
+#pod
+#pod =cut
+
 has table => (
     is       => 'ro',
+    isa      => Str,
     required => 1,
 );
 
+#pod =attr C<schema>
+#pod
+#pod The schema to which the table belongs.  Optional.
+#pod
+#pod =begin pod_coverage
+#pod
+#pod =head3 has_schema
+#pod
+#pod =end pod_coverage
+#pod
+#pod =cut
+
+
+has schema => (
+    is        => 'ro',
+    predicate => 1,
+    isa       => Str,
+);
+
+#pod =attr C<drop_table>
+#pod
+#pod If true, the table is dropped and a new one is created.
+#pod
+#pod =cut
+
 has drop_table => (
     is      => 'ro',
+    isa     => Bool,
     default => 0,
 );
 
+
+#pod =attr C<create_table>
+#pod
+#pod If true, a table will be created if it does not exist.
+#pod
+#pod =cut
+
 has create_table => (
     is      => 'ro',
+    isa     => Bool,
     default => 1,
 );
 
+#pod =attr C<primary>
+#pod
+#pod A single output column name or an array of output column names which
+#pod should be the primary key(s).  If not specified, no primary keys are
+#pod defined.
+#pod
+#pod =cut
+
 has primary => (
     is      => 'ro',
     isa     => ArrayOfStr,
@@ -64,8 +133,29 @@ has primary => (
     default => sub { [] },
 );
 
-has db_user => ( is => 'ro', default => '' );
-has db_pass => ( is => 'ro', default => '' );
+#pod =attr C<db_user>
+#pod
+#pod The name of the database user
+#pod
+#pod =cut
+
+has db_user => (
+    is      => 'ro',
+    isa     => Str,
+    default => '',
+);
+
+#pod =attr C<db_pass>
+#pod
+#pod The database password
+#pod
+#pod =cut
+
+has db_pass => (
+    is      => 'ro',
+    isa     => Str,
+    default => '',
+);
 
 
 has _sth => (
@@ -93,7 +183,7 @@ has column_defs => (
               join( ' ',
                 $field,
                 $self->output_types->{$field},
-                ( 'primary key' ) x !!( $self->primary eq $field ) );
+                ( 'primary key' )x!!( $self->primary eq $field ) );
         }
 
         return join ', ', @column_defs;
@@ -101,12 +191,28 @@ has column_defs => (
 
 );
 
+#pod =attr C<batch>
+#pod
+#pod The number of rows to write to the database at once.  This defaults to 100.
+#pod
+#pod If greater than 1, C<batch> rows are cached and then sent out in a
+#pod single transaction.  See L</Performance> for more information.
+#pod
+#pod =cut
+
 has batch => (
     is      => 'ro',
+    isa     => Int,
     default => 100,
     coerce  => sub { $_[0] > 1 ? $_[0] : 0 },
 );
 
+#pod =attr C<dbitrace>
+#pod
+#pod A trace setting passed to  L<B<DBI>>.
+#pod
+#pod =cut
+
 has dbitrace => ( is => 'ro', );
 
 has _cache => (
@@ -134,12 +240,16 @@ sub _table_exists {
 
     my $self = shift;
 
-    # ignore catalogue and schema out of sheer ignorance, and the fact
-    # that I'm not alone in that ignorance.
-
     return
-      defined $self->_dbh->table_info( '%', '%', $self->table, 'TABLE' )->fetch;
+      defined $self->_dbh->table_info( '%', $self->schema, $self->table,
+        'TABLE' )->fetch;
+}
 
+sub _fq_table_name {
+
+    my $self = shift;
+
+    join( '.', ( $self->has_schema ? ( $self->schema ) : () ), $self->table );
 }
 
 #pod =begin pod_coverage
@@ -170,9 +280,9 @@ sub setup {
       or croak( "unable to parse DSN: ", $self->dsn );
     my $dbi_driver = $dsn[1];
 
-    my $producer =  $producer{$dbi_driver} || $dbi_driver;
+    my $producer = $producer{$dbi_driver} || $dbi_driver;
 
-    my %attr       = (
+    my %attr = (
         AutoCommit => !$self->batch,
         RaiseError => 1,
     );
@@ -181,8 +291,7 @@ sub setup {
       if $dbi_driver eq 'SQLite';
 
     $self->_set__dbh(
-                     DBI->connect( $self->dsn, $self->db_user, $self->db_pass, \%attr )
-                    )
+        DBI->connect( $self->dsn, $self->db_user, $self->db_pass, \%attr ) )
       or croak( 'error connection to ', $self->dsn, "\n" );
 
     $self->_dbh->trace( $self->dbitrace )
@@ -193,7 +302,7 @@ sub setup {
         my $tr = SQL::Translator->new(
             from => sub {
                 my $schema = $_[0]->schema;
-                my $table = $schema->add_table( name => $self->table )
+                my $table = $schema->add_table( name => $self->_fq_table_name )
                   or croak $schema->error;
 
                 for my $field_name ( @{ $self->output_fields } ) {
@@ -231,7 +340,7 @@ sub setup {
 
     my $sql = sprintf(
         "insert into %s (%s) values (%s)",
-        $self->table,
+        $self->_fq_table_name,
         join( ',', @{ $self->output_fields } ),
         join( ',', ( '?' ) x @{ $self->output_fields } ),
     );
@@ -329,8 +438,8 @@ sub cleanup {
 #pod =cut
 
 
-sub say { croak }
-sub print { croak }
+sub say    { croak }
+sub print  { croak }
 sub encode { croak }
 
 with 'Data::Record::Serialize::Role::Sink';
@@ -347,7 +456,7 @@ Data::Record::Serialize::Encode::dbi - store a record in a database
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
@@ -365,6 +474,9 @@ L<B<DBI>>.
 It performs both the L<B<Data::Record::Serialize::Role::Encode>> and
 L<B<Data::Record::Serialize::Role::Sink>> roles.
 
+B<You cannot construct this directly; you must use
+L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>.>
+
 =head2 Types
 
 Field types are recognized and converted to SQL types via the following map:
@@ -380,6 +492,75 @@ C<batch> attribute) to improve performance.  Each batch is performed
 as a single transaction.  If there is an error during the transaction,
 record insertions during the transaction are I<not> rolled back.
 
+=head1 ATTRIBUTES
+
+These attributes are available in addition to the standard attributes
+defined for L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>.
+
+=head2 C<dsn>
+
+I<Required> The DBI Data Source Name (DSN) passed to B<L<DBI>>.  It
+may either be a string or an arrayref containing strings or arrayrefs,
+which should contain key-value pairs.  Elements in the sub-arrays are
+joined with C<=>, elements in the top array are joined with C<:>.  For
+example,
+
+  [ 'SQLite', { dbname => $db } ]
+
+is transformed to
+
+  SQLite:dbname=$db
+
+The standard prefix of C<dbi:> will be added if not present.
+
+=head2 C<table>
+
+I<Required> The name of the table in the database which will contain the records.
+It will be created if it does not exist.
+
+=head2 C<schema>
+
+The schema to which the table belongs.  Optional.
+
+=head2 C<drop_table>
+
+If true, the table is dropped and a new one is created.
+
+=head2 C<create_table>
+
+If true, a table will be created if it does not exist.
+
+=head2 C<primary>
+
+A single output column name or an array of output column names which
+should be the primary key(s).  If not specified, no primary keys are
+defined.
+
+=head2 C<db_user>
+
+The name of the database user
+
+=head2 C<db_pass>
+
+The database password
+
+=head2 C<batch>
+
+The number of rows to write to the database at once.  This defaults to 100.
+
+If greater than 1, C<batch> rows are cached and then sent out in a
+single transaction.  See L</Performance> for more information.
+
+=head2 C<dbitrace>
+
+A trace setting passed to  L<B<DBI>>.
+
+=begin pod_coverage
+
+=head3 has_schema
+
+=end pod_coverage
+
 =begin pod_coverage
 
 =head3 setup
@@ -408,73 +589,6 @@ record insertions during the transaction are I<not> rolled back.
 
 =end pod_coverage
 
-=head1 INTERFACE
-
-You cannot construct this directly; you must use
-L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>.
-
-=head2 Attributes
-
-These attributes are available in addition to the standard attributes
-defined for L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>.
-
-=over
-
-=item C<dsn>
-
-I<Required> The DBI Data Source Name (DSN) passed to B<L<DBI>>.  It
-may either be a string or an arrayref containing strings or arrayrefs,
-which should contain key-value pairs.  Elements in the sub-arrays are
-joined with C<=>, elements in the top array are joined with C<:>.  For
-example,
-
-  [ 'SQLite', { dbname => $db } ]
-
-is transformed to
-
-  SQLite:dbname=$db
-
-The standard prefix of C<dbi:> will be added if not present.
-
-=item db_user
-
-The name of the database user
-
-=item dbitrace
-
-A trace setting passed to  L<B<DBI>>.
-
-=item batch
-
-The number of rows to write to the database at once.  This defaults to 100.
-
-If greater than 1, C<batch> rows are cached and then sent out in a
-single transaction.  See L</Performance> for more information.
-
-=item db_pass
-
-The database password
-
-=item C<table>
-
-I<Required> The name of the table in the database which will contain the records.
-It will be created if it does not exist.
-
-=item C<drop_table>
-
-If true, the table is dropped and a new one is created.
-
-=item C<create_table>
-
-If true, a table will be created if it does not exist.
-
-=item C<primary> I<name> | I<array of names>
-
-The output name(s) of the field(s) which should be the primary key(s).
-If not specified, no primary keys are defined.
-
-=back
-
 =head1 BUGS AND LIMITATIONS
 
 You can make new bug reports, and view existing ones, through the
@@ -524,6 +638,9 @@ __END__
 #pod It performs both the L<B<Data::Record::Serialize::Role::Encode>> and
 #pod L<B<Data::Record::Serialize::Role::Sink>> roles.
 #pod
+#pod B<You cannot construct this directly; you must use
+#pod L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>.>
+#pod
 #pod =head2 Types
 #pod
 #pod Field types are recognized and converted to SQL types via the following map:
@@ -540,70 +657,7 @@ __END__
 #pod as a single transaction.  If there is an error during the transaction,
 #pod record insertions during the transaction are I<not> rolled back.
 #pod
-#pod =head1 INTERFACE
-#pod
-#pod You cannot construct this directly; you must use
-#pod L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>.
-#pod
-#pod =head2 Attributes
+#pod =head1 ATTRIBUTES
 #pod
 #pod These attributes are available in addition to the standard attributes
 #pod defined for L<B<Data::Record::Serialize-E<gt>new>|Data::Record::Serialize/new>.
-#pod
-#pod =over
-#pod
-#pod =item C<dsn>
-#pod
-#pod I<Required> The DBI Data Source Name (DSN) passed to B<L<DBI>>.  It
-#pod may either be a string or an arrayref containing strings or arrayrefs,
-#pod which should contain key-value pairs.  Elements in the sub-arrays are
-#pod joined with C<=>, elements in the top array are joined with C<:>.  For
-#pod example,
-#pod
-#pod   [ 'SQLite', { dbname => $db } ]
-#pod
-#pod is transformed to
-#pod
-#pod   SQLite:dbname=$db
-#pod
-#pod The standard prefix of C<dbi:> will be added if not present.
-#pod
-#pod
-#pod =item db_user
-#pod
-#pod The name of the database user
-#pod
-#pod =item dbitrace
-#pod
-#pod A trace setting passed to  L<B<DBI>>.
-#pod
-#pod =item batch
-#pod
-#pod The number of rows to write to the database at once.  This defaults to 100.
-#pod
-#pod If greater than 1, C<batch> rows are cached and then sent out in a
-#pod single transaction.  See L</Performance> for more information.
-#pod
-#pod =item db_pass
-#pod
-#pod The database password
-#pod
-#pod =item C<table>
-#pod
-#pod I<Required> The name of the table in the database which will contain the records.
-#pod It will be created if it does not exist.
-#pod
-#pod =item C<drop_table>
-#pod
-#pod If true, the table is dropped and a new one is created.
-#pod
-#pod =item C<create_table>
-#pod
-#pod If true, a table will be created if it does not exist.
-#pod
-#pod =item C<primary> I<name> | I<array of names>
-#pod
-#pod The output name(s) of the field(s) which should be the primary key(s).
-#pod If not specified, no primary keys are defined.
-#pod
-#pod =back
